import UIKit
import QuartzCore

//図形の描画
class GraphicsEx: UIView {
    var _context: CGContextRef! //グラフィックスコンテキスト
    
    //色の指定(2)
    func setColor(r: Int, g: Int, b: Int) {
        CGContextSetRGBFillColor(_context,
            CGFloat(r)/255.0, CGFloat(g)/255.0, CGFloat(b)/255.0, 1.0)
        CGContextSetRGBStrokeColor(_context,
            CGFloat(r)/255.0, CGFloat(g)/255.0, CGFloat(b)/255.0, 1.0)
    }
    
    //ライン幅の指定(3)
    func setLineWidth(width: Float) {
        CGContextSetLineWidth(_context, CGFloat(width))
    }
    
    //ラインの描画(3)
    func drawLine(x0: Float, y0: Float, x1: Float, y1: Float) {
        CGContextSetLineCap(_context, CGLineCap.Round)
        CGContextMoveToPoint(_context, CGFloat(x0), CGFloat(y0))
        CGContextAddLineToPoint(_context, CGFloat(x1), CGFloat(y1))
        CGContextStrokePath(_context)
    }
    
    //ポリラインの描画(3)
    func drawPolyline(x: Array<Float>, y: Array<Float>) {
        CGContextSetLineCap(_context, CGLineCap.Round)
        CGContextSetLineJoin(_context, CGLineJoin.Round)
        CGContextMoveToPoint(_context, CGFloat(x[0]), CGFloat(y[0]))
        for i in 1..<x.count {
            CGContextAddLineToPoint(_context, CGFloat(x[i]), CGFloat(y[i]))
        }
        CGContextStrokePath(_context)
    }
    
    //四角形の描画(4)
    func drawRect(x: Float, y: Float, w: Float, h: Float) {
        CGContextMoveToPoint(_context, CGFloat(x), CGFloat(y))
        CGContextAddLineToPoint(_context, CGFloat(x+w), CGFloat(y))
        CGContextAddLineToPoint(_context, CGFloat(x+w), CGFloat(y+h))
        CGContextAddLineToPoint(_context, CGFloat(x), CGFloat(y+h))
        CGContextAddLineToPoint(_context, CGFloat(x), CGFloat(y))
        CGContextAddLineToPoint(_context, CGFloat(x+w), CGFloat(y))
        CGContextStrokePath(_context)
    }
    
    //四角形の塗り潰し(4)
    func fillRect(x: Float, y: Float, w: Float, h: Float) {
        CGContextFillRect(_context,
            CGRectMake(CGFloat(x), CGFloat(y), CGFloat(w), CGFloat(h)))
    }
    
    //円の描画(5)
    func drawCircle(x: Float, y: Float, w: Float, h: Float) {
        CGContextAddEllipseInRect(_context,
            CGRectMake(CGFloat(x), CGFloat(y), CGFloat(w), CGFloat(h)))
        CGContextStrokePath(_context)
    }
    
    //円の塗り潰し(5)
    func fillCircle(x: Float, y: Float, w: Float, h: Float) {
        CGContextFillEllipseInRect(_context,
            CGRectMake(CGFloat(x), CGFloat(y), CGFloat(w), CGFloat(h)))
    }
    
    //描画
    override func drawRect(rect:  CGRect) {
        //グラフィックスコンテキストの取得(1)
        _context = UIGraphicsGetCurrentContext()
        
        //色の指定(2)
        setColor(255, g: 255, b: 255)
        
        //背景のクリア
        let w = Float(self.frame.size.width)
        let h = Float(self.frame.size.height)
        fillRect(0, y: 0, w: w, h: h)
        
        //ラインの描画(3)
        setColor(255, g: 0, b: 0)
        setLineWidth(2)
        drawLine(50, y0: 50, x1: 50, y1: 50+80)
        
        //ポリラインの描画(3)
        let dx: Array<Float> = [110, 170, 130, 190, 110]
        let dy: Array<Float> = [50, 60, 90, 100, 130]
        setColor(255, g: 0, b: 0)
        setLineWidth(3)
        drawPolyline(dx, y: dy)
        
        //四角形の描画(4)
        setColor(0, g: 255, b: 0)
        setLineWidth(1)
        drawRect(10, y: 140, w: 80, h: 80)
        
        //四角形の塗り潰し(4)
        setColor(0, g: 255, b: 0)
        setLineWidth(1)
        fillRect(110, y: 140, w: 80, h: 80)
        
        //円の描画(5)
        setColor(0, g: 0, b: 255)
        drawCircle(10, y: 240, w: 80, h: 80)
        
        //円の塗り潰し(5)
        setColor(0, g: 0, b: 255)
        fillCircle(110, y: 240, w: 80, h: 80)
    }
}
