import UIKit

//タッチイベントの処理
class TouchEx: UIView {
    var _touches = [UITouch]() //タッチオブジェクト群
    var _forceTouchAvailable = false//3D Touchが利用可能かどうか
    
    //初期化
    required init?(coder: NSCoder) {
        super.init(coder: coder)
        
        //マルチタッチの有効化(1)
        self.multipleTouchEnabled = true
        
        //ルートビューコントローラの取得(6)
        var window = UIApplication.sharedApplication().keyWindow
        if window == nil {
            window = UIApplication.sharedApplication().windows[0];
        }
        let viewCtl = window!.rootViewController
        
        //3D Touchが利用可能かどうかのチェック(7)
        _forceTouchAvailable =
            viewCtl!.traitCollection.forceTouchCapability ==
            UIForceTouchCapability.Available
    }
    
    //描画時に呼ばれる
    override func drawRect(rect: CGRect) {
        drawString("TouchEx>", x: 0, y: 20)
        
        //タッチ位置の描画(5)
        if (!_forceTouchAvailable) {
            for i in 0..<_touches.count {
                let pos = _touches[i].locationInView(self)
                let str = "touchPos[\(i)]=(\(Int(pos.x)),\(Int(pos.y)))\n"
                drawString(str, x: 0, y: 20+26+26*i)
            }
        }
        
        //タッチの強さの描画(8)
        else {
            for i in 0..<_touches.count {
                let pos = _touches[i].locationInView(self)
                let str = "touchPos[\(i)]=(\(Int(pos.x)),\(Int(pos.y))) force=\(_touches[i].force)\n"
                drawString(str, x: 0, y: 20+26+26*i)
            }
        }
    }
    
    //タッチ開始時に呼ばれる(2)
    override func touchesBegan(touches: Set<UITouch>, withEvent event: UIEvent?) {
        //タッチオブジェクトの取得(3)
        for touch in touches {
            _touches.append(touch)
        }
    
        //再描画(4)
        self.setNeedsDisplay()
    }
    
    //タッチ移動時に呼ばれる(2)
    override func touchesMoved(touches: Set<UITouch>, withEvent event: UIEvent?) {
        //再描画(4)
        self.setNeedsDisplay()
    }
    
    //タッチ終了時に呼ばれる(2)
    override func touchesEnded(touches: Set<UITouch>, withEvent event: UIEvent?) {
        //タッチオブジェクトの削除(3)
        for touch in touches {
            for i in 0..<_touches.count {
                if touch == _touches[i] {
                    _touches.removeAtIndex(i)
                    break
                }
            }
        }
    
        //再描画(4)
        self.setNeedsDisplay()
    }
    
    //タッチキャンセル時に呼ばれる(2)
    override func touchesCancelled(touches: Set<UITouch>?, withEvent event: UIEvent?) {
        //タッチオブジェクトの削除(3)
        _touches.removeAll()
    
        //再描画(4)
        self.setNeedsDisplay()
    }
    
    //文字列の描画
    func drawString(str: String, x: Int, y: Int) {
        let attrs = [NSFontAttributeName: UIFont.systemFontOfSize(24)]
        str.drawAtPoint(CGPointMake(CGFloat(x), CGFloat(y)),
            withAttributes: attrs)
    }
}
