import UIKit
import CoreLocation
import CoreBluetooth

//アドバタイズマネージャ(2)
class AdvertiseManager: NSObject, CBPeripheralManagerDelegate {
    //iBeaconのID
    var _uuid  = "" //UUID
    var _major = 1  //メジャー
    var _minor = 1  //マイナー
    
    //変数
    var _peripheralManager: CBPeripheralManager! //周辺機器マネージャ
    var _enabled = false //有効
    
    
//====================
//初期化
//====================
    //初期化(3)
    init(uuid: String, major: Int, minor: Int) {
        super.init()
        _uuid = uuid
        _major = major
        _minor = minor
        
        //周辺機器マネージャの生成
        _peripheralManager = CBPeripheralManager(
            delegate: self, queue: dispatch_get_main_queue())
        
        //自動消灯の無効化
        UIApplication.sharedApplication().idleTimerDisabled = true
    }
    
    //アドバタイズの開始(4)
    func startAdvertising() {
        if _enabled {return}
        _enabled = true
        
        //アドバタイズの開始
        let region = CLBeaconRegion(proximityUUID: NSUUID(UUIDString: _uuid)!,
            major: CLBeaconMajorValue(_major),
            minor: CLBeaconMinorValue(_minor),
            identifier: "advertizer")
        let peripheralData = region.peripheralDataWithMeasuredPower(nil)
        var dic = [String: AnyObject]()
        for (key, value) in peripheralData {
            dic[key as! String] = value
        }
        _peripheralManager.startAdvertising(dic)
    }
    
    //アドバタイズの停止(5)
    func stopAdvertising() {
        if !_enabled {return}
        _enabled = false
        
        //アドバタイズの停止
        _peripheralManager.stopAdvertising()
    }
    
    
//====================
//CBPeripheralManagerDelegate
//====================
    //周辺機器マネージャの状態更新時に呼ばれる(6)
    func peripheralManagerDidUpdateState(peripheral: CBPeripheralManager) {
    }
}