import UIKit
import CoreLocation
import CoreBluetooth
import GameKit

//iBeaconのアドバタイズ
class ViewController: UIViewController,
    BeaconRegionDelegate {
    //定数
    let BTN_START = 0 //開始
    let BTN_STOP = 1  //停止
    let POWER = -56   //ビーコンを1m離した時のRSSI値

    //iBeaconのID(1)
    let UUID = "2AEEA2B3-11D3-4432-8C16-E0B07E07785E" //UUID
    let MAJOR = 1                                     //メジャー
    let MINOR = 1                                     //マイナー
    
    //変数
    var _switch: UISwitch!                   //スイッチ
    var _output: UILabel!                    //出力
    var _beaconManager: BeaconManager!       //ビーコンマネージャ
    var _region: BeaconRegion!               //ビーコン領域
    var _advertizeManager: AdvertiseManager! //アドバタイズマネージャ
    
    //ロード完了時に呼ばれる
    override func viewDidLoad() {
        super.viewDidLoad()
        let dx: CGFloat = (UIScreen.mainScreen().bounds.size.width-320)/2
        
        //マネージャの生成
        _beaconManager = BeaconManager(delegate: self)
        _region = _beaconManager.addRegion(UUID)
        _advertizeManager = AdvertiseManager(
            uuid: UUID, major: MAJOR, minor: MINOR)
        
        //ラベルの生成
        let label = makeLabel(CGRectMake(dx+5, 20, 120, 32),
            text: "アドバタイズ", font: UIFont.systemFontOfSize(20))
        self.view.addSubview(label)
        
        //スイッチの生成
        _switch = UISwitch()
        _switch.frame = CGRectMake(dx+130, 20, 190, 32)
        _switch.on = false
        _switch.addTarget(self, action: #selector(onValueChange(_:)),
            forControlEvents: UIControlEvents.ValueChanged)
        self.view.addSubview(_switch)

        //出力の生成
        _output = makeLabel(CGRectMake(dx+5, 20+40, 310, 32),
            text: "", font: UIFont.systemFontOfSize(18))
        self.view.addSubview(_output)
    }
    
    //スイッチ変更時に呼ばれる
    func onValueChange(sender: UISwitch) {
        if sender.on {
            _advertizeManager.startAdvertising()
        } else {
            _advertizeManager.stopAdvertising()
        }
    }
    
    
//====================
//BeaconRegionDelegate(21)
//====================
    //リージョン外からリージョン内に移動した時に呼ばれる
    func onEnterRoom(sender: BeaconRegion, beacon: CLBeacon) {
        showNotification("Enter>>>>\(beacon.major)-\(beacon.minor)")
    }
    
    //リージョン内からリージョン外に移動した時に呼ばれる
    func onExitRoom(sender: BeaconRegion, beacon: CLBeacon) {
        showNotification("Exit>>>>\(beacon.major)-\(beacon.minor)")
    }
    
    //ビーコン情報の更新時に呼ばれる
    func onUpdateBeacons(sender: BeaconRegion, beacons: [CLBeacon]) {
        //出力文字列の生成
        let ms = NSMutableString()
        for (beacon) in beacons {
            var str = "\(beacon.major.intValue)-\(beacon.minor.intValue)"
            str += ",\(proximity2str(beacon.proximity))"
            str += ",rssi=\(beacon.rssi)"
            str += String(format: ",距離=%.2fm",
                rssi2distance(beacon.rssi, power: POWER))
            ms.appendString(str)
            ms.appendString("\n")
        }
        
        //出力に出力文字列を指定
        var frame = _output.frame
        frame.size = CGSizeMake(310, 9999)
        _output.frame = frame
        _output.text = ms as String
        _output.sizeToFit()
    }
    
    //エラー時に呼ばれる
    func onBeaconError(message: String) {
        print("ビーコンエラー>>>>\(message)")
    }
    
    //RSSI→距離(m)
    func rssi2distance(rssi: Int, power: Int) -> Double {
        let ratio: Double = Double(rssi) * 1.0 / Double(power)
        return 0.89976 * pow(ratio, 7.7095) + 0.111
    }
    
    
//====================
//UI
//====================
    //ラベルの生成
    func makeLabel(frame: CGRect, text: String, font: UIFont) -> UILabel {
        let label = UILabel()
        label.frame = frame
        label.text = text
        label.font = font
        label.textAlignment = NSTextAlignment.Left
        label.lineBreakMode = NSLineBreakMode.ByWordWrapping
        label.numberOfLines = 100
        return label
    }
    
    //近接→文字列
    func proximity2str(prox: CLProximity) -> String {
        if prox == CLProximity.Immediate {
            return "Immediate"
        } else if prox == CLProximity.Near {
            return "Near"
        } else if prox == CLProximity.Far {
            return "Far"
        } else {
            return "Unknown"
        }
    }
    
    //ノティフィケーションの表示
    func showNotification(message: String) {
        let notification = UILocalNotification()
        notification.fireDate = NSDate(timeInterval: 1, sinceDate: NSDate())
        notification.timeZone = NSTimeZone.defaultTimeZone()
        notification.alertBody = message
        notification.alertAction = "Open"
        notification.soundName = UILocalNotificationDefaultSoundName
        UIApplication.sharedApplication().scheduleLocalNotification(notification)
    }
}
