import UIKit
import AddressBook
import Contacts

//アドレス帳
class ViewController: UIViewController,
    UITableViewDelegate, UITableViewDataSource {
    var _tableView: UITableView! //テーブルビュー
    var _names = [String]()      //名字と名前
    var _tels = [String]()       //電話番号
    
//====================
//UI
//====================
    //ロード完了時に呼ばれる
    override func viewDidLoad() {
        super.viewDidLoad()
        
        //テーブルビューの生成
        _tableView = makeTableView(
            CGRectMake(0,20,self.view.frame.width,self.view.frame.height-20))
        self.view.addSubview(_tableView!)
        
        //アドレスの読み込み
        readAddress()
    }
    
    //テーブルビューの生成
    func makeTableView(frame: CGRect) -> UITableView {
        let tableView = UITableView()
        tableView.frame = frame
        tableView.autoresizingMask = [
            UIViewAutoresizing.FlexibleRightMargin,
            UIViewAutoresizing.FlexibleTopMargin,
            UIViewAutoresizing.FlexibleLeftMargin,
            UIViewAutoresizing.FlexibleBottomMargin,
            UIViewAutoresizing.FlexibleWidth,
            UIViewAutoresizing.FlexibleHeight]
        tableView.delegate = self
        tableView.dataSource = self
        return tableView
    }
    
    //アラートの表示(
    func showAlert(title: String?, text: String?) {
        let alert = UIAlertController(title: title, message: text,
            preferredStyle: UIAlertControllerStyle.Alert)
        alert.addAction(UIAlertAction(title: "OK",
            style: UIAlertActionStyle.Default, handler: nil))
        self.presentViewController(alert, animated: true, completion: nil)
    }
    
    //URLスキームのオープン(1)
    func openURL(urlPath: String) -> Bool {
        let url = NSURL(string: urlPath)!
        return UIApplication.sharedApplication().openURL(url)
    }
    
//====================
//アドレスの読み込み
//====================
    //アドレスの読み込み
    func readAddress() {
        _names.removeAll()
        _tels.removeAll()
        
        //CNContactStoreオブジェクトの生成(2)
        let store = CNContactStore()
    
        //アクセス許可状態の確認(3)
        let status = CNContactStore.authorizationStatusForEntityType(
            CNEntityType.Contacts)
        
        //アクセス許可を求めたことがない
        if status == CNAuthorizationStatus.NotDetermined {
            store.requestAccessForEntityType(CNEntityType.Contacts) {
                (success, error) in
                if success {
                    NSOperationQueue.mainQueue().addOperationWithBlock {
                        self.readAddress()
                    }
                } else {
                    self.showAlert(nil, text: 
                      "「設定→プライバシー→連絡先→AddressEx」をオンにしてください")
                }
            }
            return
        }
        //アクセス許可されていない
        else if status != CNAuthorizationStatus.Authorized {
            showAlert(nil, text:
                "「設定→プライバシー→連絡帳→AddressEx」をオンにしてください")
            return
        }
    
        do {
            //アドレス帳の情報取得のリクエスト(4)
            let request = CNContactFetchRequest(keysToFetch: [
                CNContactFamilyNameKey,     //名字
                CNContactGivenNameKey,      //名前
                CNContactPhoneNumbersKey])  //電話番号
            let parent = self
            try store.enumerateContactsWithFetchRequest(request, usingBlock:{
                (contact, status) -> Void in
                //アドレス帳から取得した情報の処理(5)
                parent._names.append("\(contact.familyName) \(contact.givenName)")
                if (contact.phoneNumbers.count == 0) {
                    parent._tels.append("")
                } else  {
                    let label = contact.phoneNumbers.first!
                    let pn:CNPhoneNumber = label.value as! CNPhoneNumber
                    parent._tels.append(pn.stringValue)
                }
            })
        } catch _ {
            print("エラー")
        }
    
        //テーブルビューの更新(6)
        _tableView.reloadData()
    }
    
//====================
//UITableViewDelegate
//====================
    //セルの高さの取得
    func tableView(tableView: UITableView,
        heightForRowAtIndexPath indexPath: NSIndexPath) -> CGFloat {
        return 50
    }
    
    //セルのクリック時に呼ばれる
    func tableView(tableView: UITableView,
        didSelectRowAtIndexPath indexPath: NSIndexPath) {
        //選択解除
        tableView.deselectRowAtIndexPath(
            tableView.indexPathForSelectedRow!, animated: true)
            
        //通話
        if _tels.count > 0 {
            let tel = _tels[indexPath.row]
            if !openURL("tel:\(tel)") {
                showAlert(nil, text: "通話機能がありません")
            }
        }
    }
    
    //セルの取得時に呼ばれる
    func tableView(tableView: UITableView,
        cellForRowAtIndexPath indexPath:  NSIndexPath) -> UITableViewCell {
        //テーブルのセルの生成
        let c = tableView.dequeueReusableCellWithIdentifier("table_cell")
        let cell = (c != nil) ? c! :
            UITableViewCell(style: UITableViewCellStyle.Subtitle,
                reuseIdentifier: "table_cell")
        
        //テーブルセルの設定
        cell.textLabel!.text = _names[indexPath.row]
        cell.detailTextLabel!.text = _tels[indexPath.row]
        return cell
    }
    
//====================
//UITableViewDataSource
//====================
    //セルの数取得時に呼ばれる
    func tableView(tableView: UITableView,
        numberOfRowsInSection section: Int) -> Int {
        return _names.count
    }
}
