import UIKit

//暗号化・複号化
class ViewController: UIViewController, UITextFieldDelegate {
    //定数
    let BTN_ENCRYPT = 0 //暗号化
    let BTN_DECRYPT = 1 //複号化
    
    //キーと初期化ベクトル(1)
    let KEY = "12345678901234567890123456789012" //キー
    let IV  = "12345678901234567890123456789012" //初期化ベクトル
    
    //変数
    var _textField: UITextField! //テキストフィールド
    
//====================
//UI
//====================
    //ロード完了時に呼ばれる
    override func viewDidLoad() {
        super.viewDidLoad()
        let dx: CGFloat = (UIScreen.mainScreen().bounds.size.width-320)/2
        
        //テキストフィールドの生成
        _textField = makeTextField(CGRectMake(dx+10, 20, 300, 32),
            text: "これはテストです")
        self.view.addSubview(_textField!)
        
        //暗号化ボタンの生成
        let btnEncrypt = makeButton(CGRectMake(dx+55, 62, 100, 40),
            text: "暗号化", tag: BTN_ENCRYPT)
        self.view.addSubview(btnEncrypt)
        
        //複号化ボタンの生成
        let btnDecrypt = makeButton(CGRectMake(dx+165, 62, 100, 40),
            text: "複号化", tag: BTN_DECRYPT)
        self.view.addSubview(btnDecrypt)
    }
    
    //テキストフィールドの生成
    func makeTextField(frame: CGRect, text: String) -> UITextField {
        let textField = UITextField()
        textField.frame = frame
        textField.text = text
        textField.borderStyle = UITextBorderStyle.RoundedRect
        textField.keyboardType = UIKeyboardType.Default
        textField.returnKeyType = UIReturnKeyType.Done
        textField.delegate = self
        return textField
    }
    
    //テキストボタンの生成
    func makeButton(frame: CGRect, text: String, tag: Int) -> UIButton {
        let button = UIButton(type: UIButtonType.System)
        button.frame = frame
        button.setTitle(text, forState: UIControlState.Normal)
        button.tag = tag
        button.addTarget(self, action: #selector(onClick(_:)),
            forControlEvents: UIControlEvents.TouchUpInside)
        return button
    }
    
    //ボタンクリック時に呼ばれる
    func onClick(sender: UIButton) {
        if sender.tag == BTN_ENCRYPT {
            let base64 = encrypt(_textField.text!, key: KEY, iv: IV)
            if (base64 == nil) {
                _textField.text = "エラー"
            } else {
                _textField.text = base64!
            }
        } else if (sender.tag == BTN_DECRYPT) {
            let str = decrypt(_textField.text!, key: KEY, iv: IV)
            if (str == nil) {
                _textField.text = "エラー"
            } else {
                _textField.text = str!
            }
        }
    }

//====================
//暗号化・複号化
//====================
    //文字列→バイトデータ
    func str2data(str: String) -> NSData? {
        return str.dataUsingEncoding(NSUTF8StringEncoding)
    }
    
    //バイトデータ→文字列
    func data2str(data: NSData) -> String? {
        return NSString(data: data, encoding: NSUTF8StringEncoding) as? String
    }
    
    //バイトデータ→Base64(5)
    func data2base64(data: NSData) -> String? {
        return data.base64EncodedStringWithOptions(
            NSDataBase64EncodingOptions.Encoding64CharacterLineLength)
    }
    
    //Base64→バイトデータ(5)
    func base642data(base64: String) -> NSData? {
        return NSData(base64EncodedString: base64,
            options: NSDataBase64DecodingOptions.IgnoreUnknownCharacters)
    }

    //暗号化(2)
    func encrypt(str: String, key: String, iv: String) -> String? {
        //文字列→バイトデータ
        var data = str2data(str)
        if (data == nil) {return nil}
        
        //バイトデータ→暗号化バイトデータ
        data = crypto(data!,
            op: CCOperation(kCCEncrypt), key: key, iv: iv)
        if (data == nil) {return nil}
        
        //暗号化バイトデータ→Base64
        return data2base64(data!)
    }
    
    //複号化(3)
    func decrypt(str: String, key: String, iv: String) -> String? {
        //Base64→暗号化バイトデータ
        var data = base642data(str)
        if (data == nil) {return nil}
        
        //暗号化バイトデータ→バイトデータ
        data = crypto(data!,
            op: CCOperation(kCCDecrypt), key: key, iv: iv)
        if data == nil {return nil}
        
        //バイトデータ→文字列
        return data2str(data!)
    }
    
    //暗号化と複号化(4)
    func crypto(data: NSData, op: CCOperation,
        key: String, iv: String) -> NSData? {
        
        //アルゴリズムとオプション
        let algoritm = CCAlgorithm(kCCAlgorithmAES128)
        let options = CCOptions(kCCOptionECBMode|kCCOptionPKCS7Padding)
        
        //キー
        let keyData = str2data(key)!
        let keyLength = Int(kCCKeySizeAES256)
        
        //初期化ベクトル
        let ivData = str2data(iv)!

        //出力データ
        let outLength = data.length+kCCBlockSizeAES128
        let outData = NSMutableData(length: outLength)!
        var numBytesOut: Int = 0

        //暗号化と複号化
        let cryptStatus = CCCrypt(
            op,                     //操作
            algoritm,               //アルゴリズム
            options,                //オプション
            keyData.bytes,          //キーのポインタ
            keyLength,              //キーの長さ
            ivData.bytes,           //初期化ベクトルのポインタ
            data.bytes,             //入力データのポインタ
            data.length,            //入力データの長さ
            outData.mutableBytes,   //出力データのポインタ
            outLength,              //出力データのバッファの長さ
            &numBytesOut)           //出力データの長さ
        if cryptStatus == CCCryptorStatus(kCCSuccess) {
            outData.length = numBytesOut
            return outData
        }
        return nil
    }
}
