import UIKit
import CoreImage

//顔検出
class ViewController: UIViewController,
    UINavigationControllerDelegate, UIImagePickerControllerDelegate {
    //定数
    let BTN_CAMERA = 0 //カメラ

    //変数
    var _imageView: UIImageView! //イメージビュー
    var _btnCamera: UIButton!    //カメラボタン
    
    
//====================
//UI
//====================
    //ロード完了時に呼ばれる
    override func viewDidLoad() {
        super.viewDidLoad()
        let dx: CGFloat = (UIScreen.mainScreen().bounds.size.width-320)/2
        
        //カメラボタンの生成
        _btnCamera = makeButton(CGRectMake(dx+60, 20, 200, 40),
            text: "カメラ", tag: BTN_CAMERA)
        self.view.addSubview(_btnCamera)
        
        //イメージビューの生成
        _imageView = makeImageView(CGRectMake(dx, 60, 320, 320), image: nil)
        self.view.addSubview(_imageView)
    }
    
    //ボタンクリック時に呼ばれる
    func onClick(sender: UIButton) {
        if sender.tag == BTN_CAMERA {
            openPicker(UIImagePickerControllerSourceType.Camera)
        }
    }
    
    //テキストボタンの生成
    func makeButton(frame: CGRect, text: String, tag: Int) -> UIButton {
        let button = UIButton(type: UIButtonType.System)
        button.frame = frame
        button.setTitle(text, forState: UIControlState.Normal)
        button.tag = tag
        button.addTarget(self, action: #selector(onClick(_:)),
            forControlEvents: UIControlEvents.TouchUpInside)
        return button
    }
    
    //イメージビューの生成
    func makeImageView(frame: CGRect, image: UIImage?) -> UIImageView {
        let imageView = UIImageView()
        imageView.frame = frame //領域
        imageView.image = image //イメージ
        imageView.contentMode = UIViewContentMode.ScaleAspectFit
        return imageView
    }
    
    //アラートの表示
    func showAlert(title: String?, text: String?) {
        let alert = UIAlertController(title: title, message: text,
            preferredStyle: UIAlertControllerStyle.Alert)
        alert.addAction(UIAlertAction(title: "OK",
            style: UIAlertActionStyle.Default, handler: nil))
        self.presentViewController(alert, animated: true, completion: nil)
    }
    
//====================
//イメージピッカーのオープン
//====================
    //イメージピッカーのオープン
    func openPicker(sourceType: UIImagePickerControllerSourceType) {
        //カメラとフォトアルバムの利用可能チェック
        if !UIImagePickerController.isSourceTypeAvailable(sourceType) {
            showAlert(nil, text: "利用できません")
            return
        }
    
        //イメージピッカーの生成
        let picker = UIImagePickerController()
        picker.sourceType = sourceType
        picker.delegate = self
    
        //ビューコントローラのビューを開く
        presentViewController(picker, animated: true, completion: nil)
    }
    
//====================
//UIImagePickerControllerDelegate
//====================
    //イメージピッカーのイメージ取得時に呼ばれる)
    func imagePickerController(picker: UIImagePickerController,
        didFinishPickingMediaWithInfo info: [String: AnyObject]) {
        //イメージの取得
        var image = info[UIImagePickerControllerOriginalImage]
            as! UIImage
           
        //イメージのリサイズ
        image = resizeImg(image, max: 320)
            
        //顔検出
        _imageView.image = detectFace(image)
            
        //ビューコントローラのビューを閉じる
        picker.presentingViewController?
            .dismissViewControllerAnimated(true, completion: nil)
    }
    
    //イメージピッカーのキャンセル時に呼ばれる(
    func imagePickerControllerDidCancel(picker: UIImagePickerController) {
        //ビューコントローラのビューを閉じる
        picker.presentingViewController?
            .dismissViewControllerAnimated(true, completion: nil)
    }
    
    
//====================
//顔検出
//====================
    //顔検出
    func detectFace(image: UIImage) -> UIImage {
        //CIイメージの生成(1)
        let ciImage = CIImage(CGImage: image.CGImage!)
        
        //顔の検出(2)
        let orientation = NSNumber(integer: exifOrientation(image))
        let ciDetector = CIDetector(ofType: CIDetectorTypeFace,
            context: nil,
            options: [
                CIDetectorAccuracy: CIDetectorAccuracyHigh, //検出精度高
                CIDetectorImageOrientation:orientation]     //画像の向き
        )
        
        //検出の実行(3)
        let features = ciDetector.featuresInImage(ciImage)
        
        //イメージ生成の開始(5)
        UIGraphicsBeginImageContext(image.size)
        let context = UIGraphicsGetCurrentContext()
        image.drawInRect(CGRectMake(0,0,image.size.width,image.size.height))
        
        //検出結果の表示(4)
        for feature in features {
            //顔
            let face = feature as! CIFaceFeature
            var faceRect = face.bounds
            faceRect.origin.y = image.size.height-faceRect.origin.y-faceRect.size.height
            CGContextSetStrokeColorWithColor(context,UIColor.whiteColor().CGColor)
            CGContextStrokeRect(context,faceRect)
            
            //口
            if (face.hasMouthPosition) {
                let mouseRect = CGRectMake(
                    face.mouthPosition.x-5,
                    image.size.height-face.mouthPosition.y-5,10,10)
                CGContextSetStrokeColorWithColor(context,UIColor.whiteColor().CGColor)
                CGContextStrokeRect(context,mouseRect)
            }

            //左目
            if (face.hasLeftEyePosition) {
                let leftEyeRect = CGRectMake(
                    face.leftEyePosition.x-5,
                    image.size.height-face.leftEyePosition.y-5,10,10)
                CGContextSetStrokeColorWithColor(context,UIColor.whiteColor().CGColor)
                CGContextStrokeRect(context,leftEyeRect)
            }
            
            //右目
            if (face.hasRightEyePosition) {
                let rightEyeRect = CGRectMake(
                    face.rightEyePosition.x-5,
                    image.size.height-face.rightEyePosition.y-5,10,10)
                CGContextSetStrokeColorWithColor(context, UIColor.whiteColor().CGColor)
                CGContextStrokeRect(context,rightEyeRect)
            }
        }
        
        //イメージ生成の完了(5)
        let result = UIGraphicsGetImageFromCurrentImageContext()
        UIGraphicsEndImageContext()
        return result
    }
    
    //写真の向きの取得
    func exifOrientation(image: UIImage) -> Int {
        var orientation: Int = 0
        switch (image.imageOrientation) {
        case UIImageOrientation.Up:
            orientation = 1
        case UIImageOrientation.Down:
            orientation = 3
        case UIImageOrientation.Left:
            orientation = 8
        case UIImageOrientation.Right:
            orientation = 6
        case UIImageOrientation.UpMirrored:
            orientation = 2
        case UIImageOrientation.DownMirrored:
            orientation = 4
        case UIImageOrientation.LeftMirrored:
            orientation = 5
        case UIImageOrientation.RightMirrored:
            orientation = 7
        }
        return orientation
    }
    
    //イメージのリサイズ(6)
    func resizeImg(image: UIImage, max: CGFloat) -> UIImage {
        //サイズ
        let imageW = image.size.width
        let imageH = image.size.height
        
        //倍率の計算
        let scale = imageW > imageH ? max/imageH : max/imageW
        
        //リサイズ
        let resizedSize = CGSizeMake(imageW*scale, imageH*scale)
        UIGraphicsBeginImageContext(CGSizeMake(imageW*scale, imageH*scale))
        image.drawInRect(CGRectMake(0, 0, resizedSize.width, resizedSize.height))
        let resizedImage = UIGraphicsGetImageFromCurrentImageContext()
        UIGraphicsEndImageContext()
        return resizedImage
    }
}