import UIKit
import AVFoundation

//音声合成
class ViewController: UIViewController {
    //定数
    let BTN_JA = 0 //日本語
    let BTN_EN = 1 //英語
    
    //変数
    var _textField: UITextField! //テキストフィールド
    
//====================
//UI
//====================
    //ロード完了時に呼ばれる
    override func viewDidLoad() {
        super.viewDidLoad()
        let dx: CGFloat = (UIScreen.mainScreen().bounds.size.width-320)/2
        
        //テキストフィールドの生成
        _textField = makeTextField(CGRectMake(dx+10, 20, 300, 32),
            text: "Hello")
        self.view.addSubview(_textField)
        
        
        //日本語ボタンの生成
        let btnJa = makeButton(CGRectMake(dx+60, 70, 200, 40),
            text: "日本語で読み上げ", tag: BTN_JA)
        self.view.addSubview(btnJa)
        
        //英語ボタンの生成
        let btnEn = makeButton(CGRectMake(dx+60, 120, 200, 40),
            text: "英語で読み上げ", tag: BTN_EN)
        self.view.addSubview(btnEn)
    }
    
    //ボタンクリック時に呼ばれる
    func onClick(sender: UIButton) {
        if sender.tag == BTN_JA {
            speech(_textField.text!, language: "ja-JP")
        } else if sender.tag == BTN_EN {
            speech(_textField.text!, language: "en-US")
        }
    }
    
    //テキストフィールドの初期化
    func makeTextField(frame: CGRect, text: String) -> UITextField {
        let textField = UITextField()
        textField.frame = frame
        textField.text = text
        textField.borderStyle = UITextBorderStyle.RoundedRect
        textField.keyboardType = UIKeyboardType.Default
        textField.returnKeyType = UIReturnKeyType.Done
        return textField
    }
    
    //ボタンの生成
    func makeButton(frame: CGRect, text: String, tag: Int) -> UIButton {
        let button = UIButton(type: UIButtonType.System)
        button.frame = frame
        button.setTitle(text, forState: UIControlState.Normal)
        button.tag = tag
        button.addTarget(self, action: #selector(onClick(_:)),
            forControlEvents: UIControlEvents.TouchUpInside)
        return button
    }
    
    
//====================
//音声合成
//====================
    //声の再生
    func speech(text: String, language: String) {
        //読み上げ情報の生成(1)
        let utterance = AVSpeechUtterance(string: text)
        utterance.rate = AVSpeechUtteranceMinimumSpeechRate  //読み上げ速度
        utterance.volume = 1                                 //ボリューム
        utterance.voice = AVSpeechSynthesisVoice(language:language) //声(2)
        
        //読み上げの実行
        let synthesizer = AVSpeechSynthesizer()
        synthesizer.speakUtterance(utterance)
    }
}
