import UIKit
import StoreKit

//非消耗型のアプリ内課金
class ViewController: UIViewController,
    SKProductsRequestDelegate,
    SKPaymentTransactionObserver {
    let BTN_PAYMENT = 0 //購入
    let BTN_RESTORE = 1 //リストア
    let PRODUCT_ID = "NonConsumableProduct" //プロダクトID(1)
    
    var _label: UILabel!       //ラベル
    var _paymentBtn: UIButton! //購入ボタン
    var _restoreBtn: UIButton! //リストアボタン
    var _product: SKProduct!   //プロダクト
    var _hasItem: Bool = false //アイテム保持
    
    //ロード完了時に呼ばれる
    override func viewDidLoad() {
        super.viewDidLoad()
        let dx: CGFloat = (UIScreen.mainScreen().bounds.size.width-320)/2
        
        //ラベルとボタンの生成
        _label = makeLabel(CGRectMake(dx+20, 40, 280, 60),
            text:"プロダクト情報の取得中…")
        self.view.addSubview(_label)
        _paymentBtn = makeButton(CGRectMake(dx+60, 110, 200, 40),
            text: "購入", tag: BTN_PAYMENT)
        _paymentBtn.enabled = false
        self.view.addSubview(_paymentBtn)
        _restoreBtn = makeButton(CGRectMake(dx+60, 155, 200, 40),
            text: "リストア", tag: BTN_RESTORE)
        _restoreBtn.enabled = false
        self.view.addSubview(_restoreBtn)
        
        //アイテムの保持チェック
        let pref = NSUserDefaults.standardUserDefaults()
        _hasItem = pref.boolForKey("item")
        if (_hasItem) {
            _label.text = "購入済み"
            return
        }
        
        //プロダクト情報の取得
        if self.checkPurchase() {
            self.startProductsRequest()
        }
    }
    
    //アプリ内課金が利用可能かどうかの取得(2)
    func checkPurchase() -> Bool {
        if !SKPaymentQueue.canMakePayments() {
            self.showAlert(nil, text:
                "「設定→一般→機能制限→App内課金」をONにしてください")
            return false
        }
        return true
    }
    
    //プロダクト情報取得の開始(3)
    func startProductsRequest() {
        let set = Set([PRODUCT_ID])
        let productsRequest = SKProductsRequest(productIdentifiers:set)
        productsRequest.delegate = self
        productsRequest.start()
    }

    //プロダクト情報取得の成功時に呼ばれる(4)
    func productsRequest(request: SKProductsRequest,
        didReceiveResponse response: SKProductsResponse) {
        //無効なプロダクトをチェック
        if (response.invalidProductIdentifiers.count > 0) {
            self.showAlert(nil, text: "プロダクトIDが不正です")
            return
        }
        
        //プロダクト情報の表示
        if (response.products.count == 1) {
            _paymentBtn.enabled = true
            _restoreBtn.enabled = true
            _product = response.products.first!
            _label.text = "タイトル:\(_product.localizedTitle)\n価格:\(_product.price)\n説明:\(_product.localizedDescription)"
        }
    }
    
    //プロダクト情報取得の失敗時に呼ばれる(4)
    func request(request: SKRequest, didFailWithError error: NSError) {
        self.showAlert(nil, text: "プロダクト情報取得が失敗しました")
    }
    
    //購入処理の開始(5)
    func startPayment() {
        _paymentBtn.enabled = false
        _restoreBtn.enabled = false
        let payment = SKPayment(product: _product)
        SKPaymentQueue.defaultQueue().addTransactionObserver(self)
        SKPaymentQueue.defaultQueue().addPayment(payment)
    }
    
    //トランザクションの状態変更時に呼ばれる(6)
    func paymentQueue(queue: SKPaymentQueue,
        updatedTransactions transactions: [SKPaymentTransaction]) {
        for trans:SKPaymentTransaction in transactions {
            //購入処理中
            if trans.transactionState ==
                SKPaymentTransactionState.Purchasing {
                //インジケータなどの処理を記述
            }
            //購入成功
            else if trans.transactionState ==
                SKPaymentTransactionState.Purchased {
                //アイテム購入処理(機能制限解除など)を記述
                if trans.payment.productIdentifier == PRODUCT_ID {
                    _label.text = "購入済み"
                    _hasItem = true
                    
                    //保存
                    let pref = NSUserDefaults.standardUserDefaults()
                    pref.setBool(_hasItem, forKey: "item")
                    pref.synchronize()
                }
                
                //トランザクション完了
                queue.finishTransaction(trans)
            }
            //購入失敗(ユーザーキャンセル含む)
            else if trans.transactionState ==
                SKPaymentTransactionState.Failed {
                //ユーザーキャンセル以外
                if trans.error?.code != SKErrorCode.PaymentCancelled.rawValue {
                    self.showAlert(nil, text:trans.error?.description)
                }
                    
                //トランザクション完了
                queue.finishTransaction(trans)
            }
            //リストア完了(9)
            else if trans.transactionState ==
                SKPaymentTransactionState.Restored {
                //アイテム購入処理(機能制限解除など)を記述
                if trans.payment.productIdentifier == PRODUCT_ID {
                    _label.text = "購入済み"
                    _hasItem = true
                    
                    //保存
                    let pref = NSUserDefaults.standardUserDefaults()
                    pref.setBool(_hasItem, forKey: "item")
                    pref.synchronize()
                }
              
                //トランザクション完了
                queue.finishTransaction(trans)
            }
            //その他のエラー
            else {
                //トランザクション完了
                queue.finishTransaction(trans)
            }
        }   
    }    
    
    //トランザクションの終了時に呼ばれる(7)
    func paymentQueue(queue: SKPaymentQueue,
        removedTransactions transactions: [SKPaymentTransaction]) {
        if (!_hasItem) {
            _paymentBtn.enabled = true
            _restoreBtn.enabled = true
        }
    }
    
    //リストア処理の開始(8)
    func startRestore() {
        _paymentBtn.enabled = false
        _restoreBtn.enabled = false
        SKPaymentQueue.defaultQueue().addTransactionObserver(self)
        SKPaymentQueue.defaultQueue().restoreCompletedTransactions()
    }
    
    //全リストア処理の完了時に呼ばれる(9)
    func paymentQueueRestoreCompletedTransactionsFinished(
        queue: SKPaymentQueue) {
        self.showAlert(nil, text: "リストアが完了しました")
    }
    
    //リストア処理の失敗時に呼ばれる(9)
    func paymentQueue(queue: SKPaymentQueue,
        restoreCompletedTransactionsFailedWithError error: NSError) {
        self.showAlert(nil, text: "リストアが失敗しました")
    }
    
    //ボタンクリック時に呼ばれる
    func onClick(sender: UIButton) {
        if (sender.tag == BTN_PAYMENT) {
            self.startPayment()
        } else if (sender.tag == BTN_RESTORE) {
            self.startRestore()
        }
    }
    
    //ラベルの生成
    func makeLabel(frame: CGRect, text: String) -> UILabel {
        let label = UILabel()
        label.frame = frame
        label.text = text
        label.font = UIFont.systemFontOfSize(16)
        label.textAlignment = NSTextAlignment.Center
        label.lineBreakMode = NSLineBreakMode.ByWordWrapping
        label.numberOfLines = 3
        return label
    }
    
    //ボタンの生成
    func makeButton(frame: CGRect, text: String, tag: Int) -> UIButton {
        let button = UIButton(type: UIButtonType.System)
        button.frame = frame
        button.setTitle(text, forState: UIControlState.Normal)
        button.tag = tag
        button.addTarget(self, action: #selector(onClick(_:)),
            forControlEvents: UIControlEvents.TouchUpInside)
        return button
    }
    
    //アラートの表示
    func showAlert(title: String?, text: String?) {
        let alert = UIAlertController(title: title, message: text,
            preferredStyle: UIAlertControllerStyle.Alert)
        alert.addAction(UIAlertAction(title: "OK",
            style: UIAlertActionStyle.Default, handler: nil))
        self.presentViewController(alert, animated: true, completion: nil)
    }
}
