import UIKit
import StoreKit

//消費型のアプリ内課金
class ViewController: UIViewController,
    SKProductsRequestDelegate,
    SKPaymentTransactionObserver {
    let BTN_PAYMENT = 0 //購入
    let PRODUCT_ID = "ConsumableProduct" //プロダクトID(1)
    
    var _label: UILabel!       //ラベル
    var _paymentBtn: UIButton! //購入ボタン
    var _product: SKProduct!   //プロダクト
    
    //ロード完了時に呼ばれる
    override func viewDidLoad() {
        super.viewDidLoad()
        let dx: CGFloat = (UIScreen.mainScreen().bounds.size.width-320)/2
        
        //ラベルとボタンの生成
        _label = makeLabel(CGRectMake(dx+20, 40, 280, 60),
            text:"プロダクト情報の取得中…")
        self.view.addSubview(_label)
        _paymentBtn = makeButton(CGRectMake(dx+60, 110, 200, 40),
            text: "購入", tag: BTN_PAYMENT)
        _paymentBtn.enabled = false
        self.view.addSubview(_paymentBtn)
        
        //プロダクト情報の取得
        if self.checkPurchase() {
            self.startProductsRequest()
        }
        
        //アプリ起動時にも独自サーバに未送信レシートを送信(6)
        self.accessServer({
            (success: Bool) in
            if success {
                self.showAlert(nil,
                    text:"アイテムが追加されました")
            }
        })
    }
    
    //アプリ内課金が利用可能かどうかの取得
    func checkPurchase() -> Bool {
        if !SKPaymentQueue.canMakePayments() {
            self.showAlert(nil, text:
                "「設定→一般→機能制限→App内課金」をONにしてください")
            return false
        }
        return true
    }
    
    //プロダクト情報取得の開始
    func startProductsRequest() {
        let set = Set([PRODUCT_ID])
        let productsRequest = SKProductsRequest(productIdentifiers:set)
        productsRequest.delegate = self
        productsRequest.start()
    }

    //プロダクト情報取得の成功時に呼ばれる
    func productsRequest(request: SKProductsRequest,
        didReceiveResponse response: SKProductsResponse) {
        //無効なプロダクトをチェック
        if (response.invalidProductIdentifiers.count > 0) {
            self.showAlert(nil, text: "プロダクトIDが不正です")
            return
        }
        
        //プロダクト情報の表示
        if (response.products.count == 1) {
            _paymentBtn.enabled = true
            _product = response.products.first!
            _label.text = "タイトル:\(_product.localizedTitle)\n価格:\(_product.price)\n説明:\(_product.localizedDescription)"
        }
    }
    
    //プロダクト情報取得の失敗時に呼ばれる
    func request(request: SKRequest, didFailWithError error: NSError) {
        self.showAlert(nil, text: "プロダクト情報取得が失敗しました")
    }
    
    //購入処理の開始
    func startPayment() {
        _paymentBtn.enabled = false
        let payment = SKPayment(product: _product)
        SKPaymentQueue.defaultQueue().addTransactionObserver(self)
        SKPaymentQueue.defaultQueue().addPayment(payment)
    }
    
    //トランザクションの状態変更時に呼ばれる
    func paymentQueue(queue: SKPaymentQueue,
        updatedTransactions transactions: [SKPaymentTransaction]) {
         
        for trans:SKPaymentTransaction in transactions {
            //購入処理中
            if trans.transactionState ==
                SKPaymentTransactionState.Purchasing {
                //インジケータなどの処理を記述
            }
            //購入成功
            else if trans.transactionState ==
                SKPaymentTransactionState.Purchased {
                //トランザクション完了
                queue.finishTransaction(trans)
                
                //レシートの取得(2)
                let receiptURL = NSBundle.mainBundle().appStoreReceiptURL
                let receiptData = NSData(contentsOfURL:receiptURL!)
                
                //レシートの保存(3)
                let storage = NSUserDefaults.standardUserDefaults()
                var receipts = storage.arrayForKey("receipts")
                if (receiptData != nil) {
                    if (receipts == nil) {
                        storage.setObject([receiptData!], forKey:"receipts")
                    } else {
                        receipts!.append(receiptData!)
                        storage.setObject(receipts, forKey:"receipts")
                    }
                    storage.synchronize()
                }
                
                //独自サーバに未送信レシートを送信(4)
                self.accessServer({
                    (success: Bool) in
                    if success {
                        self.showAlert(nil,
                            text:"アイテムが追加されました")
                    }
                })
            }
            //購入失敗(ユーザーキャンセル含む)
            else if trans.transactionState ==
                SKPaymentTransactionState.Failed {
                //ユーザーキャンセル以外
                if trans.error?.code != SKErrorCode.PaymentCancelled.rawValue {
                    self.showAlert(nil, text:trans.error?.description)
                }
                    
                //トランザクション完了
                queue.finishTransaction(trans)
            }
            //その他のエラー
            else {
                //トランザクション完了
                queue.finishTransaction(trans)
            }
        }   
    }    
    
    //トランザクションの終了時に呼ばれる
    func paymentQueue(queue: SKPaymentQueue,
        removedTransactions transactions: [SKPaymentTransaction]) {
        _paymentBtn.enabled = true
    }
    
    //独自サーバに未送信レシートを送信(4)
    func accessServer(completion: (Bool) -> Void) {
        dispatch_async(dispatch_get_global_queue(
            DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), {
            let storage = NSUserDefaults.standardUserDefaults()
            var receipts = storage.arrayForKey("receipts")
        
            var result = false
            if (receipts != nil) {
                //独自サーバに未送信レシートを送信
                for i in 0..<receipts!.count {
                    let receiptData = receipts![i] as! NSData
                    let base64 = receiptData.base64EncodedStringWithOptions(
                        NSDataBase64EncodingOptions.Encoding64CharacterLineLength)
            
                    //独自サーバに未送信レシートを送信する処理をここに記述
                    //※今回のサンプルでは実際の通信処理は省略しています
                    print("送信>>>>\(base64)")
                }
                result = true
        
                //通信成功時にストレージの削除
                if (result) {
                    storage.removeObjectForKey("receipts")
                    storage.synchronize()
                }
            }
    
            //結果を返す
            dispatch_async(dispatch_get_main_queue(), {
                completion(result)
            })
        })
    }
    
    //ボタンクリック時に呼ばれる
    func onClick(sender: UIButton) {
        if (sender.tag == BTN_PAYMENT) {
            self.startPayment()
        }
    }
    
    //ラベルの生成
    func makeLabel(frame: CGRect, text: String) -> UILabel {
        let label = UILabel()
        label.frame = frame
        label.text = text
        label.font = UIFont.systemFontOfSize(16)
        label.textAlignment = NSTextAlignment.Center
        label.lineBreakMode = NSLineBreakMode.ByWordWrapping
        label.numberOfLines = 3
        return label
    }
    
    //ボタンの生成
    func makeButton(frame: CGRect, text: String, tag: Int) -> UIButton {
        let button = UIButton(type: UIButtonType.System)
        button.frame = frame
        button.setTitle(text, forState: UIControlState.Normal)
        button.tag = tag
        button.addTarget(self, action: #selector(onClick(_:)),
            forControlEvents: UIControlEvents.TouchUpInside)
        return button
    }
    
    //アラートの表示
    func showAlert(title: String?, text: String?) {
        let alert = UIAlertController(title: title, message: text,
            preferredStyle: UIAlertControllerStyle.Alert)
        alert.addAction(UIAlertAction(title: "OK",
            style: UIAlertActionStyle.Default, handler: nil))
        self.presentViewController(alert, animated: true, completion: nil)
    }
}
