import UIKit
import StoreKit

//自動更新購読型のアプリ内課金
class ViewController: UIViewController,
    SKRequestDelegate,
    SKProductsRequestDelegate,
    SKPaymentTransactionObserver {
    let BTN_PAYMENT = 0 //購入
    let BTN_RESTORE = 1 //リストア
    let PRODUCT_ID = "AutoRenewableProduct" //プロダクトID(1)
    
    var _label: UILabel!                          //ラベル
    var _paymentBtn: UIButton!                    //購入ボタン
    var _restoreBtn: UIButton!                    //リストアボタン
    var _product: SKProduct!                      //プロダクト
    var _expirationDate: NSDate!                  //購入期限
    var _receiptRequest: SKReceiptRefreshRequest! //レシート更新リクエスト
    
    //ロード完了時に呼ばれる
    override func viewDidLoad() {
        super.viewDidLoad()
        let dx: CGFloat = (UIScreen.mainScreen().bounds.size.width-320)/2
        
        //ラベルとボタンの生成
        _label = makeLabel(CGRectMake(dx+20, 40, 280, 60),
            text:"プロダクト情報の取得中…")
        self.view.addSubview(_label)
        _paymentBtn = makeButton(CGRectMake(dx+60, 110, 200, 40),
            text: "購入", tag: BTN_PAYMENT)
        _paymentBtn.enabled = false
        self.view.addSubview(_paymentBtn)
        _restoreBtn = makeButton(CGRectMake(dx+60, 155, 200, 40),
            text: "リストア", tag: BTN_RESTORE)
        _restoreBtn.enabled = false
        self.view.addSubview(_restoreBtn)
        
        //起動時に有効期限をチェック(6)
        let pref = NSUserDefaults.standardUserDefaults()
        let time = pref.integerForKey("expirationDate")
        if (time != 0) {
            _expirationDate = NSDate(timeIntervalSince1970: NSTimeInterval(time))
            if _expirationDate.compare(NSDate()) ==
                NSComparisonResult.OrderedAscending {
                _expirationDate = nil
                
                //レシートの更新
                _receiptRequest = SKReceiptRefreshRequest()
                _receiptRequest.delegate = self
                _receiptRequest.start()
                return
            }
        }
        if (_expirationDate != nil) {
            _label.text = "購入済み \(self.date2str(_expirationDate))"
            return
        }
        
        //プロダクト情報の取得
        if self.checkPurchase() {
            self.startProductsRequest()
        }
    }
    
    //日付→文字列
    func date2str(date: NSDate?) -> String {
        if date == nil {return ""}
        
        //オブジェクト型の変数の定義
        let calendar: NSCalendar =
            NSCalendar(calendarIdentifier:
            NSCalendarIdentifierGregorian)!
        
        //取得情報フラグの準備
        let flags: NSCalendarUnit = [
            NSCalendarUnit.Year,   //年
            NSCalendarUnit.Month,  //月
            NSCalendarUnit.Day,    //日
            NSCalendarUnit.Hour,   //時
            NSCalendarUnit.Minute] //分

        //日付コンポーネントの取得
        let comps = calendar.components(flags, fromDate: date!)
        
        //年月日の取得
        let year = comps.year     //年
        let month = comps.month   //月
        let day = comps.day       //日
        let hour = comps.hour     //時
        let minute = comps.minute //分
        return "\(year)/\(month)/\(day) \(hour):\(minute)"
    }
    
    //アプリ内課金が利用可能かどうかの取得
    func checkPurchase() -> Bool {
        if !SKPaymentQueue.canMakePayments() {
            self.showAlert(nil, text:
                "「設定→一般→機能制限→App内課金」をONにしてください")
            return false
        }
        return true
    }
    
    //プロダクト情報取得の開始
    func startProductsRequest() {
        let set = Set([PRODUCT_ID])
        let productsRequest = SKProductsRequest(productIdentifiers:set)
        productsRequest.delegate = self
        productsRequest.start()
    }

    //プロダクト情報取得の成功時に呼ばれる
    func productsRequest(request: SKProductsRequest,
        didReceiveResponse response: SKProductsResponse) {
        //無効なプロダクトをチェック
        if (response.invalidProductIdentifiers.count > 0) {
            self.showAlert(nil, text: "プロダクトIDが不正です")
            return
        }
        
        //プロダクト情報の表示
        if (response.products.count == 1) {
            _paymentBtn.enabled = true
            _restoreBtn.enabled = true
            _product = response.products.first!
            _label.text = "タイトル:\(_product.localizedTitle)\n価格:\(_product.price)\n説明:\(_product.localizedDescription)"
        }
    }
    
    //プロダクト情報取得の失敗時に呼ばれる
    func request(request: SKRequest, didFailWithError error: NSError) {
        self.showAlert(nil, text: "プロダクト情報取得が失敗しました")
    }
    
    //購入処理の開始
    func startPayment() {
        _paymentBtn.enabled = false
        _restoreBtn.enabled = false
        let payment = SKPayment(product: _product)
        SKPaymentQueue.defaultQueue().addTransactionObserver(self)
        SKPaymentQueue.defaultQueue().addPayment(payment)
    }
    
    //トランザクションの状態変更時に呼ばれる
    func paymentQueue(queue: SKPaymentQueue,
        updatedTransactions transactions: [SKPaymentTransaction]) {
        for trans:SKPaymentTransaction in transactions {
            //購入処理中
            if trans.transactionState ==
                SKPaymentTransactionState.Purchasing {
                //インジケータなどの処理を記述
            }
            //購入成功
            else if trans.transactionState ==
                SKPaymentTransactionState.Purchased {
                //アイテム購入処理(機能制限解除など)を記述
                if trans.payment.productIdentifier == PRODUCT_ID {
                    //有効期限の取得
                    _expirationDate = self.getExpirationDate()
                    
                    //保存
                    let pref = NSUserDefaults.standardUserDefaults()
                    if _expirationDate == nil {
                        pref.removeObjectForKey("expirationDate")
                    } else {
                        pref.setInteger(Int(_expirationDate!.timeIntervalSince1970),
                            forKey: "expirationDate")
                    }
                    pref.synchronize()
                }
                
                //トランザクション完了
                queue.finishTransaction(trans)
            }
            //購入失敗(ユーザーキャンセル含む)
            else if trans.transactionState ==
                SKPaymentTransactionState.Failed {
                //ユーザーキャンセル以外
                if trans.error?.code != SKErrorCode.PaymentCancelled.rawValue {
                    self.showAlert(nil, text:trans.error?.description)
                }
                    
                //トランザクション完了
                queue.finishTransaction(trans)
            }
            //リストア完了(9)
            else if trans.transactionState ==
                SKPaymentTransactionState.Restored {
                //アイテム購入処理(機能制限解除など)を記述
                if trans.payment.productIdentifier == PRODUCT_ID {
                    //有効期限の取得
                    _expirationDate = self.getExpirationDate()
                    
                    //保存
                    let pref = NSUserDefaults.standardUserDefaults()
                    if _expirationDate == nil {
                        pref.removeObjectForKey("expirationDate")
                    } else {
                        pref.setInteger(Int(_expirationDate!.timeIntervalSince1970),
                            forKey: "expirationDate")
                    }
                    pref.synchronize()
                }
              
                //トランザクション完了
                queue.finishTransaction(trans)
            }
            //その他のエラー
            else {
                //トランザクション完了
                queue.finishTransaction(trans)
            }
        }   
    }
    
    //レシートから有効期限の取得
    func getExpirationDate() -> NSDate? {
        //レシート情報の取得(2)
        let receiptURL = NSBundle.mainBundle().appStoreReceiptURL
        if !verifyReceiptAtPath(receiptURL!.path) {return nil}
        let receipt = dictionaryWithAppStoreReceipt(receiptURL!.path)
        if (receipt == nil) {return nil}
        print("receipt>>>>\(receipt)")
    
        //レシートから有効期限の取得(3)
        let inAppArray = receipt["InApp"] as! NSArray
        var time: NSTimeInterval = 0
        for (inApp) in inAppArray {
            //SubExpDateの最大値の取得
            let subExpDate = inApp["SubExpDate"] as! String
            if (subExpDate == "") {continue}
            let date = self.rfc33392date(subExpDate)
            let t = date!.timeIntervalSince1970
            if (time < t) {time = t}
        }
        if (time == 0) {return nil}
        let date = NSDate(timeIntervalSince1970: time)
        
        //有効期限切れの時はnilを返す(4)
        if date.compare(NSDate()) ==
            NSComparisonResult.OrderedAscending {
            return nil
        }
        return date
    }
    
    //RFC3339→NSDate(5)
    func rfc33392date(rfc3339: String) -> NSDate? {
        let dateFormatter = NSDateFormatter()
        let locale = NSLocale(localeIdentifier:"en_US_POSIX")
        dateFormatter.locale = locale
        dateFormatter.dateFormat = "yyyy'-'MM'-'dd'T'HH':'mm':'ss'Z'"
        dateFormatter.timeZone = NSTimeZone(forSecondsFromGMT:0)
        return dateFormatter.dateFromString(rfc3339)
    }
    
    //トランザクションの終了時に呼ばれる
    func paymentQueue(queue: SKPaymentQueue,
        removedTransactions transactions: [SKPaymentTransaction]) {
        //表示の更新
        if (_expirationDate == nil) {
            _paymentBtn.enabled = true
            _restoreBtn.enabled = true
        } else {
            _label.text = "購入済み \(self.date2str(_expirationDate))"
        }
    }
    
    //リストア処理の開始
    func startRestore() {
        _paymentBtn.enabled = false
        _restoreBtn.enabled = false
        SKPaymentQueue.defaultQueue().addTransactionObserver(self)
        SKPaymentQueue.defaultQueue().restoreCompletedTransactions()
    }
    
    //全リストア処理の完了時に呼ばれる
    func paymentQueueRestoreCompletedTransactionsFinished(
        queue: SKPaymentQueue) {
        self.showAlert(nil, text: "リストアが完了しました")
    }
    
    //リストア処理の失敗時に呼ばれる
    func paymentQueue(queue: SKPaymentQueue,
        restoreCompletedTransactionsFailedWithError error: NSError) {
        self.showAlert(nil, text: "リストアが失敗しました")
    }
    
    //リクエスト完了時に呼ばれる(6)
    func requestDidFinish(request: SKRequest) {
        //レシート更新リクエスト以外は無処理
        if (request != _receiptRequest) {return}
        
        //有効期限の取得
        _expirationDate = self.getExpirationDate()
                    
        //保存
        let pref = NSUserDefaults.standardUserDefaults()
        if _expirationDate == nil {
            pref.removeObjectForKey("expirationDate")
        } else {
            pref.setInteger(Int(_expirationDate!.timeIntervalSince1970),
                forKey: "expirationDate")
        }
        pref.synchronize()
        
        //表示の更新
        if (_expirationDate == nil) {
            _paymentBtn.enabled = true
            _restoreBtn.enabled = true
        } else {
            _label.text = "購入済み \(self.date2str(_expirationDate))"
        }
    }
    
    //ボタンクリック時に呼ばれる
    func onClick(sender: UIButton) {
        if (sender.tag == BTN_PAYMENT) {
            self.startPayment()
        } else if (sender.tag == BTN_RESTORE) {
            self.startRestore()
        }
    }
    
    //ラベルの生成
    func makeLabel(frame: CGRect, text: String) -> UILabel {
        let label = UILabel()
        label.frame = frame
        label.text = text
        label.font = UIFont.systemFontOfSize(16)
        label.textAlignment = NSTextAlignment.Center
        label.lineBreakMode = NSLineBreakMode.ByWordWrapping
        label.numberOfLines = 3
        return label
    }
    
    //ボタンの生成
    func makeButton(frame: CGRect, text: String, tag: Int) -> UIButton {
        let button = UIButton(type: UIButtonType.System)
        button.frame = frame
        button.setTitle(text, forState: UIControlState.Normal)
        button.tag = tag
        button.addTarget(self, action: #selector(onClick(_:)),
            forControlEvents: UIControlEvents.TouchUpInside)
        return button
    }
    
    //アラートの表示
    func showAlert(title: String?, text: String?) {
        let alert = UIAlertController(title: title, message: text,
            preferredStyle: UIAlertControllerStyle.Alert)
        alert.addAction(UIAlertAction(title: "OK",
            style: UIAlertActionStyle.Default, handler: nil))
        self.presentViewController(alert, animated: true, completion: nil)
    }
}
